library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.common_pkg.all;
--use work.alu_pkg.all;
--use work.gpm_pkg.all;

package extension_pkg is

constant EXTWORDL : integer := log2c(4);
constant BYTEADDR : integer := log2c(4);
constant PCOUNT   : integer := 3;
constant EXTWORDS : integer := EXTWORDL + BYTEADDR;

subtype ext_addrid_t  is std_logic_vector(gp_register_t'high - EXTWORDS downto 0);
subtype ext_addr_t    is std_logic_vector((gp_register_t'high-BYTEADDR) downto 0);    
subtype paddr_t is std_logic_vector(log2c(PCOUNT)-1 downto 0);   

        type extmod_rec is record
                sel   : std_logic;
                wr_en : std_logic;
                byte_en : byte_en_t;
                data : gp_register_t;
                addr : ext_addr_t;     
        end record; 


type status_rec is record
		zero : std_logic;
		oflo : std_logic;
		sign : std_logic;
		carry : std_logic;
end record;


-- Addressen der bis jetzt vorhanden extensions
constant EXT_UART_ADDR:   ext_addrid_t := x"0000200";
constant EXT_7SEG_ADDR:   ext_addrid_t := x"0000201";
constant EXT_INT_ADDR:	  ext_addrid_t := x"0000202";
constant EXT_IMP_ADDR:	  ext_addrid_t := x"0000203";
-- dummy addressen
constant EXT_EXTMEM_ADDR: ext_addrid_t := x"FFFFFFB";
constant EXT_TIMER_ADDR:  ext_addrid_t := x"FFFFFFC";
constant EXT_AC97_ADDR:   ext_addrid_t := x"FFFFFFD";
constant EXT_GPMP_ADDR:    ext_addrid_t := x"FFFFFFF";

component extension_gpm is
        --some modules won't need all inputs/outputs
	generic	(
			-- active reset value
			RESET_VALUE : std_logic
			);
	port(
		--System inputs
			clk :   in std_logic;
			reset : in std_logic;
		-- general extension interface			
			ext_reg  : in extmod_rec;
			 data_out : out gp_register_t;
		-- Input
			psw_nxt : in status_rec;
			paddr 	: in paddr_t;
			pinc	: in std_logic;
			pwr_en  : in std_logic;
		-- Ouput
			 psw     : out status_rec;
			 pval	 : out gp_register_t;
			 pval_nxt : out gp_register_t
		);
 end component extension_gpm;

component extension_interrupt is
        --some modules won't need all inputs/outputs
	generic	(
			-- active reset value
			RESET_VALUE : std_logic
			);
	port(
		--System inputs
			clk :   in std_logic;
			reset : in std_logic;
		-- general extension interface			
			ext_reg  : in extmod_rec;
			data_out : out gp_register_t;
		-- Input
			uart_int : in std_logic;
		-- Ouput
			int_req : out interrupt_t
		);
 end component extension_interrupt;


end package extension_pkg;
